;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;
;;;;;;	INSTANCE-BASED LEARNING ACT-R model in order to show the developmental transitions in reasoning about false beliefs of others. 
;;;;;;  These stand from a child's reasoning from his/her own point of view (zero-order) to taking into consideration 
;;;;;;  an other agent’s beliefs (first-order) and later to taking into consideration an other agent’s beliefs about 
;;;;;;  other agents’ beliefs (second-order). 
;;;;;;	Copyright (C) 2013  Burcu Arslan
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;
;;;;;;	The model code is written by Burcu Arslan (b.arslan@rug.nl)
;;;;;;	Department of Artificial Intelligence
;;;;;;	University of Groningen
;;;;;;	The Netherlands
;;;;;;
;;;;;;	Last change to model code:
;;;;;;		Date: 2015.10.07
;;;;;;		Who: Burcu Arslan
;;;;;;
;;;;;;	Instructions on using this model:
;;;;;;		1. Make sure you have ACT-R 6.0
;;;;;;		2. Call (fbt) if you run the model for 1 child doing the experiment 1 time
;;;;;;          3. Call (do-fbt) if you run the model for 1 child doing the experiment 100 times
;;;;;;          4. Call (do-fbt-n N) if you run the model for N children doing the experiment 100 times
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



(defvar *response* nil) ;; global variable that stores model output
(defvar *model* nil)
(defvar *hold-responses* nil)  ;; global variable that stores model output



;; do experiment 1 time    
(defun fbt (&optional who)

(goal-focus goal)
  (if (eq who 'human)
      (setf *model* nil)
         (setf *model* t)
   )
        (run 30 :real-time t) 
)


;; do experiment 100 times 
(defun do-fbt ()  	
	(dotimes (i 100)
            (setf *response* nil)					 		
			(fbt)
               (push *response* *hold-responses*)
         )                       
)

;; do experiment nx100 times 
(defun do-fbt-n (n)    
	  (dotimes (i n)	
              (setf *hold-responses* nil)                	
                        (reset)				 		
			(do-fbt)
                        (write-to-file (concatenate 'string "dat-" (write-to-string i)) (nreverse *hold-responses*))
;(setf *hold-response* nil)
         )
)

;; write results to a file
(defun write-to-file (name lst)
	(with-open-file 
		(out 
			(ensure-directories-exist 
				(merge-pathnames 
					(make-pathname :name name :directory '(:relative "ACTR_model_output_2015") :type "txt")  
					"~/")
                        )
			:direction :output :if-does-not-exist :create :if-exists :supersede
                 )
			(dolist (line lst)
				(format out "~{~a~^~t~}~%" line))
         )
)

	

(clear-all) ;; clear model settings

(define-model FBT_2   ;name of the model

;;model parameters
(sgp 

	:esc t ; enable sub-symbolic level
	:ol t ;  enable optimised learning; default
	:bll .5 ; set base level learning equation d variable to 0.5; default value
	:rt  -5 ;retrieval threshold set to very low number in order to make sure that the model always retrieve the story facts
	:ans .1 ; instantaneous noise
        :v  t   ; trace detail
        :pas nil ; permanent noise
        :act low  ; activation trace parameter
        :sact low ;save activation trace
)


;chunk-types of the model
(chunk-type story subject negation verb object location time type self-ref ref level)  ; for the story facts
(chunk-type zaman t0 t1 t2 t3 t4)     ; for the time sequences
(chunk-type goal state type output)   ; for the goal state, type of the where question (action), and for the answer that the model produces
(chunk-type reasoning level)          ; for the level of reasoning
 
(add-dm

;story fact chunks
(not isa chunk) (action isa chunk) (perception isa chunk) (subject isa chunk) (question isa chunk) 
(location isa chunk) (verb isa chunk) (sub isa chunk) (object isa chunk) (negation isa chunk)
(put isa chunk) (see isa chunk) (type isa chunk) (murat isa chunk) (ayla isa chunk) (chocolate isa chunk)

;goal state chunks
(start isa chunk) (start1 isa chunk) (start2 isa chunk) (start3 isa chunk)
(reason isa chunk) (reason1 isa chunk) (stop isa chunk) (reasoning isa chunk) 
(increment-reasoning isa chunk) (answer isa chunk)(answer1 isa chunk)(answer2 isa chunk)   

;temporal order chunk
 (t0 ISA zaman t0 1 t1 2 t2 3)


;Model's knowledge representations about the story facts
 
 (s1 ISA story subject murat verb put object chocolate location "drawer" time 1 type action self-ref s1) 
 (s2 ISA story subject ayla verb put object chocolate location "toybox" time 2 type action self-ref s2) 
 (s3 ISA story subject murat verb see time 2 type perception self-ref s3 ref s2) 
 (s4 ISA story subject ayla negation not verb see time 2 type perception self-ref s4 ref s3) 
 (s5 ISA story subject mother verb put object chocolate location "tvstand" time 3 type action self-ref s5) 

;model;s knowledge about the level of reasoning (zero-order reasoning)
 (reasoning-zero isa reasoning level 1)


;assign this goal at the beginning of the trial
 (goal isa goal state start type action)
)

;base level activation for zero-order reasoning
(set-base-levels (reasoning-zero 6))



; These production rules are for the decision making about the level of reasoning

(p choose-reasoning
   =goal>
      ISA         goal
      state       reasoning
  ?retrieval>
      state       free
      buffer      empty
  =imaginal>
      ISA         story
==>
   +retrieval>
      ISA        reasoning
  =imaginal>
   =goal>
      state       answer
)
(p increment-reasoning    
   =goal>
      ISA         goal
      state       increment-reasoning
   =retrieval>
      ISA        reasoning
      level      =level
  ?imaginal>
      state       free
      buffer      empty
==>
    !bind! =levelup (+ =level 1)
   =retrieval>
      level      =levelup
   =goal>
      state        stop
    -retrieval>
)

(p do-not-increment-reasoning-at-level-3    
   =goal>
      ISA         goal
      state       increment-reasoning
   =imaginal>
      ISA        story
      level      3 
==>
   =goal>
      state        stop
    +retrieval>
      ISA         reasoning
      level       3
) 
  
; These production rules are the same for every level of reasoning

(p where-question-in-general
   =goal>
      ISA         goal
      state       start
      type        =type         
==>
   =goal>
      state      start1
   +retrieval> 
      ISA         story
      type        =type                   
)
             
(p retrieved-story-check-time
   =goal>
      ISA         goal
      state       start1
  ?imaginal>
      state       free
   =retrieval>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
==>
   =goal>
      state      start2
  +retrieval>
     ISA        zaman
  +imaginal>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
)
(p retrieved-story-by-type-time-t0
   =goal>
      ISA         goal
      state       start2
  =imaginal>
      ISA         story
      verb        =verb
      object      =obj
      time        =time
      type        =type  
  =retrieval>
     ISA        zaman
     t0         =time
     t1         =time1
     t2         =time2
==>
   =goal>
      state      start3
  +retrieval>
      ISA         story
      object      =obj
      time        =time2
      type        =type
)
(p retrieved-story-by-type-time-t00
   =goal>
      ISA         goal
      state       start2
  =imaginal>
      ISA         story
      verb        =verb
      object      =obj
      time        =time1
      type        =type  
  =retrieval>
     ISA        zaman
     t0         =time
     t1         =time1
     t2         =time2
==>
   =goal>
      state      start3
  +retrieval>
      ISA         story
      object      =obj
      time        =time2
      type        =type
)
(p requested-story-by-type-time-t1
   =goal>
      ISA         goal
      state       start3
  =retrieval>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
   ?imaginal>    
==>
   =goal>
      state      reasoning
  +imaginal>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
)

(p retrieved-story-by-type-time-t1
   =goal>
      ISA         goal
      state       start2
  =imaginal>
      ISA         story
      time        =time 
  =retrieval>
     ISA        zaman
     t2         =time
==>
   =goal>
      state       reasoning
  =imaginal>

)

; Now the model should decide to use a strategy

(p answer-zero-order     ; if the model retrieves zero-order reasoning it gives a correct answer for the question 'Where is the chocolate now?' not for the second-order false belief question
   =goal>
      ISA         goal
      state       answer
   =retrieval>
      ISA        reasoning
      level       1
  =imaginal>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
==>   
   =goal>
      state       increment-reasoning
      output      =loc
      !output!    (=loc)
   =retrieval>
   -imaginal>
!safe-eval! (push 0 *response*)  ; put 0 as a strategy level  
!safe-eval! (push (sdp (reasoning-zero reasoning-zero-0 reasoning-zero-0-1) :name :activation :base-level :creation-Time :reference-Count) *response*)  ; this function writes the name of values of the reasoning chunks
)


(p where-reason-first-order     ; if the model retrieves first-order reasoning it gives a correct answer for the question 'Where will Murat look for the chocolate?' not for the second-order false belief question
   =goal>
      ISA         goal
      state       answer
   =retrieval>
      ISA        reasoning
      level       2
   =imaginal>
      ISA         story
      subject     =subject
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      level       nil
     self-ref     =s    
==>
   =imaginal>
      level       2
   +retrieval>            
      ISA        story
     ;time         2
     subject     murat
     type        perception
   =goal>
      state       reason1
)
(p keep-first-order         
   =goal>
      ISA         goal
      state       reason1
   =imaginal>
      ISA         story
  ;    subject     =subject
      object      =obj
  ;    location    =loc
  ;    negation    nil
   ;   time        =time
      type        =type
  ;   self-ref     =s
      level       =level
   =retrieval>
      ISA        story
     subject     =subj
     negation    nil           
     time        =time
     ref         =s
  ; - type        =type
==>
    =imaginal>
      object      =obj
      type        =type
      level       =level
   +retrieval>
     ISA        story
     self-ref   =s
    =goal>
     state      reason333
)     
(p answer-first-order         
   =goal>
      ISA         goal
      state       reason333
   =imaginal>
      ISA         story
      level       =level
   =retrieval>
      ISA        story
     subject     =subj
     negation    nil
     location    =loc       
     time        =time
    ; ref         =s
  ; - type        =type
==>
   +retrieval>
     ISA        reasoning
     level      =level
   =goal>
     state       increment-reasoning
     output      =loc
    !output!    (=loc)
!safe-eval! (push 1 *response*)  ; put 1 as a strategy level
!safe-eval! (push (sdp (reasoning-zero reasoning-zero-0 reasoning-zero-0-1) :name :activation :base-level :creation-Time :reference-Count) *response*)  ; this function writes the name of values of the reasoning chunks  
)

(p where-reason-second-order-first-order    ; if the model retrieves second-order reasoning it gives a correct answer for the second-order false belief  question 'Where does Ayla think that Murat will look for the chocolate?'
   =goal>
      ISA         goal
      state       answer
   =retrieval>
      ISA        reasoning
      level       3
   =imaginal>
      ISA         story
      subject     =subject
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
     self-ref     =s
     level        nil    
==>
   =imaginal>
      level       3
   +retrieval>            
      ISA        story
     time         2
     subject     murat
     type        perception
   =goal>
      state       reason33
)
(p where-reason-second-order-keep-first-order         
   =goal>
      ISA         goal
      state       reason33
   =imaginal>
      ISA         story
      object      =obj
      level       =level
      time        =time
      type        =type1
   =retrieval>
      ISA        story
     subject     =subj
     negation    nil           
     time        =time1
     self-ref    =s
     type        =type
==>
    =imaginal>
      object      =obj
      subject     =subj
      type        =type1
      level       =level
   +retrieval>
     ISA          story
     subject      ayla
     time         =time1
     type         =type
     ref          =s
    =goal>
     state      reason334
)
(p reason-second-order
   =goal>
      ISA         goal                       
      state       reason334
   =imaginal>
      ISA         story
      subject     =subj
      object      =obj
      type        =type
      level       =level
   =retrieval>
      ISA        story
     subject     =subj1          
     time        =time
    negation     not 
      type        =type1     
==>
   =goal>
      state       answer1
  +retrieval>
      ISA         story
      subject    =subj
      object      =obj
      type       =type
      time        1
   =imaginal>
 ;     subject     =subj
  ;    object      =obj
  ;    type        =type
  ;    level       =level      
)

(p answer-second-order
   =goal>
      ISA         goal                       
      state       answer1
   =imaginal>
      ISA         story
      subject     =subj
      object      =obj
      type        =type
      level        3          
   =retrieval>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
==>
   =imaginal>
   =goal>
     state       increment-reasoning
      output      =loc
   !output!    (=loc)
!safe-eval! (push 2 *response*) ; put 2 as a strategy level
!safe-eval! (push (sdp (reasoning-zero reasoning-zero-0 reasoning-zero-0-1) :name :activation :base-level :creation-Time :reference-Count) *response*)  ; this function writes the name activation values of the reasoning chunks
)
)

                                                            