;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;
;;;;;;	REINFORCEMENT LEARNING ACT-R model in order to show the developmental transitions in reasoning about false beliefs of others. 
;;;;;;  These stand from a child's reasoning from his/her own point of view (zero-order) to taking into consideration 
;;;;;;  an other agent’s beliefs (first-order) and later to taking into consideration an other agent’s beliefs about 
;;;;;;  other agents’ beliefs (second-order).
;;;;;; 
;;;;;;	Copyright (C) 2016  Burcu Arslan
;;;;;;  code:LISP
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;
;;;;;;	The model code is written by Burcu Arslan (b.arslan@rug.nl)
;;;;;;	Department of Artificial Intelligence
;;;;;;	University of Groningen
;;;;;;	The Netherlands
;;;;;;
;;;;;;	Last change to model code:
;;;;;;		Date: 2016.01.14
;;;;;;		Who: Burcu Arslan
;;;;;;
;;;;;;	Instructions on using this model:
;;;;;;		1. Make sure you have ACT-R 6.0
;;;;;;		2. Call (fbt) if you run the model for 1 child doing the experiment 1 time
;;;;;;          3. Call (do-fbt) if you run the model for 1 child doing the experiment 100 times
;;;;;;          4. Call (do-fbt-n N) if you run the model for N children doing the experiment 100 times
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



(defvar *response* nil) ;; global variable that stores model output
(defvar *model* nil)
(defvar *hold-responses* nil)  ;; global variable that stores model output



;; do experiment 1 time    
(defun fbt (&optional who)

(goal-focus goal)
  (if (eq who 'human)
      (setf *model* nil)
         (setf *model* t)
   )
        (run 30 :real-time t) 
)


;; do experiment 100 times 
(defun do-fbt ()  	
	(dotimes (i 100)
            (setf *response* nil)					 		
			(fbt)
               (push *response* *hold-responses*)
         )                       
)

;; do experiment nx100 times 
(defun do-fbt-n (n)    
	  (dotimes (i n)	
              (setf *hold-responses* nil)                	
                        (reset)				 		
			(do-fbt)
                        (write-to-file (concatenate 'string "dat-" (write-to-string i)) (nreverse *hold-responses*))
;(setf *hold-response* nil)
         )
)

;; write results to a file
(defun write-to-file (name lst)
	(with-open-file 
		(out 
			(ensure-directories-exist 
				(merge-pathnames 
					(make-pathname :name name :directory '(:relative "ACTR_model_output_2015") :type "txt")  
					"~/")
                        )
			:direction :output :if-does-not-exist :create :if-exists :supersede
                 )
			(dolist (line lst)
				(format out "~{~a~^~t~}~%" line))
         )
)

	

(clear-all) ;; clear model settings

(define-model FBT_2   ;name of the model

;;model parameters
(sgp 

	:esc t ; enable sub-symbolic level
	:ol t ;  enable optimised learning; default
	:bll .5 ; set base level learning equation d variable to 0.5; default value
	:rt  -5 ;retrieval threshold set to very low number in order to make sure that the model always retrieve the story facts
	:ans .1 ; instantaneous noise
        :v  t   ; trace detail
        :pas nil ; permanent noise
        :act low  ; activation trace parameter
        :sact low ;save activation trace
        :egs 3 ;utility noise
        :ul t ;utility learning is turned on
        :ult t ;turn on the trace for the utilities
)


;chunk-types of the model
(chunk-type story subject negation verb object location time type self-ref ref)  ; for the story facts
(chunk-type zaman t0 t1 t2 t3 t4)     ; for the time sequences
(chunk-type goal state type output)   ; for the goal state, type of the where question (action), and for the answer that the model produces
 
(add-dm

;story fact chunks
(not isa chunk) (action isa chunk) (perception isa chunk) (subject isa chunk) (question isa chunk) 
(location isa chunk) (verb isa chunk) (sub isa chunk) (object isa chunk) (negation isa chunk)
(put isa chunk) (see isa chunk) (type isa chunk) (murat isa chunk) (ayla isa chunk) (chocolate isa chunk)

;goal state chunks
(start isa chunk) (start1 isa chunk) (start2 isa chunk) (start3 isa chunk)
(reason isa chunk) (reason1 isa chunk) (stop isa chunk) (reasoning isa chunk) 
(increment-reasoning isa chunk) (answer isa chunk)(answer1 isa chunk)(answer2 isa chunk)   

;temporal order chunk
 (t0 ISA zaman t0 1 t1 2 t2 3)


;Model's knowledge representations about the story facts
 
 (s1 ISA story subject murat verb put object chocolate location "drawer" time 1 type action self-ref s1) 
 (s2 ISA story subject ayla verb put object chocolate location "toybox" time 2 type action self-ref s2) 
 (s3 ISA story subject murat verb see time 2 type perception self-ref s3 ref s2) 
 (s4 ISA story subject ayla negation not verb see time 2 type perception self-ref s4 ref s3) 
 (s5 ISA story subject mother verb put object chocolate location "tvstand" time 3 type action self-ref s5) 



;assign this goal at the beginning of the trial
 (goal isa goal state start type action)
)




; These production rules are the same for each level of reasoning (e.g., zero-order, first-order, second-order)

(p where-question-in-general
   =goal>
      ISA         goal
      state       start
      type        =type         
==>
   =goal>
      state      start1
   +retrieval> 
      ISA         story
      type        =type                   
)
             
(p retrieved-story-check-time
   =goal>
      ISA         goal
      state       start1
  ?imaginal>
      state       free
   =retrieval>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
==>
   =goal>
      state      start2
  +retrieval>
     ISA        zaman
  +imaginal>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
)
(p retrieved-story-by-type-time-t0
   =goal>
      ISA         goal
      state       start2
  =imaginal>
      ISA         story
      verb        =verb
      object      =obj
      time        =time
      type        =type  
  =retrieval>
     ISA        zaman
     t0         =time
     t1         =time1
     t2         =time2
==>
   =goal>
      state      start3
  +retrieval>
      ISA         story
      object      =obj
      time        =time2
      type        =type
)
(p retrieved-story-by-type-time-t00
   =goal>
      ISA         goal
      state       start2
  =imaginal>
      ISA         story
      verb        =verb
      object      =obj
      time        =time1
      type        =type  
  =retrieval>
     ISA        zaman
     t0         =time
     t1         =time1
     t2         =time2
==>
   =goal>
      state      start3
  +retrieval>
      ISA         story
      object      =obj
      time        =time2
      type        =type
)
(p retrieved-story-by-type-time-t00000
   =goal>
      ISA         goal
      state       start2
  =imaginal>
      ISA         story
      verb        =verb
      object      =obj
      time        =time2
      type        =type  
  =retrieval>
     ISA        zaman
     t0         =time
     t1         =time1
     t2         =time2
==>
   =goal>
      state      answer
  =imaginal>
)

(p requested-story-by-type-time-t1
   =goal>
      ISA         goal
      state       start3
  =retrieval>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        3
      type        =type
      self-ref    =s
   ?imaginal>
      state       free    
==>
   =goal>
      state      answer
  +imaginal>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        3
      type        =type
      self-ref    =s
)

(p answer-zero-order   ; if the model uses zero-order reasoning it gives a correct answer for the question 'Where is the chocolate now?' not for the second-order false belief question
   =goal>
      ISA         goal
      state       answer
  =imaginal>
      ISA         story
      subject     =subject
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
     self-ref     =s  
==>
=goal>
      state       increment-reasoning
      output      =loc
      !output!    (=loc)
 ;  =retrieval>
 ;  -imaginal>
!safe-eval! (push 0 *response*)  ; this function writes the name of the production rule to a variable as an answer  
!safe-eval! (push (spp  (answer-zero-order where-reason-first-order where-reason-second-order-first-order) :name :utility :u :at :reward) *response*)  ; this function writes the name of values of the reasoning chunks
)


(p where-reason-first-order     ; if the model uses first-order reasoning it gives a correct answer for the question 'Where will Murat look for the chocolate?' not for the second-order false belief question
   =goal>
      ISA         goal
      state       answer
   =imaginal>
      ISA         story
      subject     =subject
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
     self-ref     =s    
==>
   +retrieval>            
      ISA        story
     subject     murat
     type        perception
   =imaginal>
   =goal>
      state       reason1
)
(p keep-first-order         
   =goal>
      ISA         goal
      state       reason1
   =imaginal>
      ISA         story
      object      =obj
      type        =type
   =retrieval>
      ISA        story
     subject     =subj
     negation    nil           
     time        =time
     ref         =s
==>
    =imaginal>
      object      =obj
      type        =type
   +retrieval>
     ISA        story
     self-ref   =s
    =goal>
     state      reason333
)     
(p answer-first-order         
   =goal>
      ISA         goal
      state       reason333
   =imaginal>
      ISA         story
   =retrieval>
      ISA        story
     subject     =subj
     negation    nil
     location    =loc       
     time        =time
==>
   =goal>
     state       increment-reasoning
     output      =loc
    !output!    (=loc)
!safe-eval! (push 1 *response*)  ; this function writes the name of the production rule to a variable as an answer  
!safe-eval! (push (spp (answer-zero-order where-reason-first-order where-reason-second-order-first-order) :name :utility :u :at :reward) *response*)  ; this function writes the name of values of the reasoning chunks  
)

(p where-reason-second-order-first-order    ; if the model uses second-order reasoning it gives the correct answer for the second-order false belief  question 'Where does Ayla think that Murat will look for the chocolate?'
   =goal>
      ISA         goal
      state       answer
   =imaginal>
      ISA         story
      subject     =subject
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
     self-ref     =s 
==>
   +retrieval>            
      ISA        story
     time         2
     subject     murat
     type        perception
   =imaginal>
   =goal>
      state       reason33
)
(p where-reason-second-order-keep-first-order         
   =goal>
      ISA         goal
      state       reason33
   =imaginal>
      ISA         story
      object      =obj
      time        =time
      type        =type1
   =retrieval>
      ISA        story
     subject     =subj
     negation    nil           
     time        =time1
     self-ref    =s
     type        =type
==>
    =imaginal>
      object      =obj
      subject     =subj
      type        =type1
   +retrieval>
     ISA          story
     subject      ayla
     time         =time1
     type         =type
     ref          =s
    =goal>
     state      reason334
)
(p reason-second-order
   =goal>
      ISA         goal                       
      state       reason334
   =imaginal>
      ISA         story
      subject     =subj
      object      =obj
      type        =type
   =retrieval>
      ISA        story
     subject     =subj1          
     time        =time
    negation     not 
      type        =type1     
==>
   =goal>
      state       answer1
  +retrieval>
      ISA         story
      subject    =subj
      object      =obj
      type       =type
      time        1
   =imaginal> 
)

(p answer-second-order
   =goal>
      ISA         goal                       
      state       answer1
   =imaginal>
      ISA         story
      subject     =subj
      object      =obj
      type        =type     
   =retrieval>
      ISA         story
      subject     =subj
      verb        =verb
      object      =obj
      location    =loc
      negation    nil
      time        =time
      type        =type
      self-ref    =s
==>
   =imaginal>
   =goal>
     state       increment-reasoning
      output      =loc
   !output!    (=loc)
!safe-eval! (push 2 *response*) ; this function writes the name of the production rule to a variable as an answer 
!safe-eval! (push (spp (answer-zero-order where-reason-first-order where-reason-second-order-first-order) :name :utility :u :at :reward) *response*)  ; this function writes the name activation values of the reasoning chunks
)

;initial utility values
(spp answer-zero-order :u 100)
(spp where-reason-first-order :u 25)
(spp where-reason-second-order-first-order :u 5)

; reward values
(spp answer-zero-order :reward 0)
(spp answer-first-order :reward 0)
(spp answer-second-order :reward 20)
)

                                                            